/*
 * File:   IRMPmain.c
 * Author: pcxxxxxxx
 *
 * Created on December 30, 2013, 2:56 PM
 * IF the bootloader is used:
 *  use linker option for the bootloader,--rom=default,-7D00-7FFF
 * realy want 32Mhz which is ~8Mhz in the avr world, or the int may take to long.
 * 1 July 2014, used irmp-trunk
 * put same changes in as last year, compiles with less warnings.
 *
 * TODO: not tested yet!
 * WARN: no linker option in this project for my bootloader! if we burn it as is,
 * the bl is gone!!! so have the pickit ready for the 1st test.
 */

// PIC18F4520 Configuration Bit Settings, use same as bootloader.asm !!

#include <xc.h>

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

// CONFIG1H
#pragma config OSC = INTIO67    // Oscillator Selection bits (Internal oscillator block, port function on RA6 and RA7)
#pragma config FCMEN = ON       // Fail-Safe Clock Monitor Enable bit (Fail-Safe Clock Monitor enabled)
#pragma config IESO = OFF       // Internal/External Oscillator Switchover bit (Oscillator Switchover mode disabled)

// CONFIG2L
#pragma config PWRT = OFF       // Power-up Timer Enable bit (PWRT disabled)
#pragma config BOREN = SBORDIS  // Brown-out Reset Enable bits (Brown-out Reset enabled in hardware only (SBOREN is disabled))
#pragma config BORV = 3         // Brown Out Reset Voltage bits (Minimum setting)

// CONFIG2H
#pragma config WDT = OFF        // Watchdog Timer Enable bit (WDT disabled (control is placed on the SWDTEN bit))
#pragma config WDTPS = 32768    // Watchdog Timer Postscale Select bits (1:32768)

// CONFIG3H
#pragma config CCP2MX = PORTC   // CCP2 MUX bit (CCP2 input/output is multiplexed with RC1)
#pragma config PBADEN = ON      // PORTB A/D Enable bit (PORTB<4:0> pins are configured as analog input channels on Reset)
#pragma config LPT1OSC = OFF    // Low-Power Timer1 Oscillator Enable bit (Timer1 configured for higher power operation)
#pragma config MCLRE = ON       // MCLR Pin Enable bit (MCLR pin enabled; RE3 input pin disabled)

// CONFIG4L
#pragma config STVREN = ON      // Stack Full/Underflow Reset Enable bit (Stack full/underflow will cause Reset)
#pragma config LVP = OFF        // Single-Supply ICSP Enable bit (Single-Supply ICSP disabled)
#pragma config XINST = OFF      // Extended Instruction Set Enable bit (Instruction set extension and Indexed Addressing mode disabled (Legacy mode))

// CONFIG5L
#pragma config CP0 = OFF        // Code Protection bit (Block 0 (000800-001FFFh) not code-protected)
#pragma config CP1 = OFF        // Code Protection bit (Block 1 (002000-003FFFh) not code-protected)
#pragma config CP2 = OFF        // Code Protection bit (Block 2 (004000-005FFFh) not code-protected)
#pragma config CP3 = OFF        // Code Protection bit (Block 3 (006000-007FFFh) not code-protected)

// CONFIG5H
#pragma config CPB = OFF        // Boot Block Code Protection bit (Boot block (000000-0007FFh) not code-protected)
#pragma config CPD = OFF        // Data EEPROM Code Protection bit (Data EEPROM not code-protected)

// CONFIG6L
#pragma config WRT0 = OFF       // Write Protection bit (Block 0 (000800-001FFFh) not write-protected)
#pragma config WRT1 = OFF       // Write Protection bit (Block 1 (002000-003FFFh) not write-protected)
#pragma config WRT2 = OFF       // Write Protection bit (Block 2 (004000-005FFFh) not write-protected)
#pragma config WRT3 = OFF       // Write Protection bit (Block 3 (006000-007FFFh) not write-protected)

// CONFIG6H
#pragma config WRTC = OFF       // Configuration Register Write Protection bit (Configuration registers (300000-3000FFh) not write-protected)
#pragma config WRTB = OFF       // Boot Block Write Protection bit (Boot block (000000-0007FFh) not write-protected)
#pragma config WRTD = OFF       // Data EEPROM Write Protection bit (Data EEPROM not write-protected)

// CONFIG7L
#pragma config EBTR0 = OFF      // Table Read Protection bit (Block 0 (000800-001FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR1 = OFF      // Table Read Protection bit (Block 1 (002000-003FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR2 = OFF      // Table Read Protection bit (Block 2 (004000-005FFFh) not protected from table reads executed in other blocks)
#pragma config EBTR3 = OFF      // Table Read Protection bit (Block 3 (006000-007FFFh) not protected from table reads executed in other blocks)

// CONFIG7H
#pragma config EBTRB = OFF      // Boot Block Table Read Protection bit (Boot block (000000-0007FFh) not protected from table reads executed in other blocks)


#define _XTAL_FREQ 32000000UL // 32mhz -> befehlstakt -> 8mhz
#define FOSC _XTAL_FREQ
#define FCY  FOSC / 4UL // Cycle Frequenz

#define BAUDRATE 19200UL
#define BRG (( FCY / 16 / BAUDRATE ) -1UL)

#include <stdio.h>
#include <stdlib.h>

#include "irmp.h"

void interrupt high_priority high_isr(void) {
    LATD0 = 1; // blink LED

    if (TMR2IF) {
        TMR2IF = 0; // clear Timer 0 interrupt flag

         irmp_ISR();

    }
    LATD0 = 0; // blink LED
}

void interrupt low_priority low_isr(void) {
    LATD1 = 1; // blink LED

    if (RCIF) {
#ifdef INVERT_UART
        if (FERR && RC7)
#else
        if (FERR && PORTCbits.RC7 == 0)
#endif
        {
            // receiving BREAK state, soft reboot back into Bootloader mode.
            RESET();
        }

        TXREG = RCREG; // echo received serial data back
    }
    LATD1 = 0; // blink LED
}

void putch(unsigned char byte) {
    while (TXIF == 0) {
        // spin until done transmitting
    }

    TXREG = byte;
}

// wait , no echo
unsigned char getch(void) // Zeichen empfangen ohne Echo
{
    // overrun?
    if (RCSTAbits.OERR) {
        RCSTAbits.CREN = 0; // reset receiver logic
        RCSTAbits.CREN = 1; // enable

        return 0; // return something then
    }

    // this is to enter bootloader mode if break has been asserted!!
    if (FERR && PORTCbits.RC7 == 0) {
        // receiving BREAK state, soft reboot back into Bootloader mode.
        RESET();
    }

    while (!RCIF); // wait blocks!

    return RCREG;
}

int main(int argc, char** argv) {

    IRMP_DATA irmp_data;

    // speed: 32 Mhz
    OSCCON = 0x70;
    OSCTUNEbits.PLLEN = 1; // PLL enable

    // USART init
    unsigned char dummy;
    SPBRG = FOSC / (16ul * BAUDRATE) - 1;
    TXSTA = 0b00100100; // xmitter on, high Baudrate
    TRISCbits.TRISC7 = 1; // receiver pin RX=RC7 is input
    TRISCbits.TRISC6 = 1; // xmit pin TX=RC6 is input
    RCSTA = 0b10010000; // Receiver on
    dummy = RCREG; // empty Receiver

#ifdef INVERT_UART
    LATC6 = 0; // drive TXD pin low for RS-232 IDLE state
#else
    LATC6 = 1; // drive TXD pin high for RS-232 IDLE state
#endif
    TRISC6 = 0;

    // for speed measure: looks like it is @ 32Mhz, OK
    TRISBbits.RB0 = 0;
    LATBbits.LATB0 = 0;

    ADCON1bits.PCFG = 0b1111;  // RE1 must be declared as Digital !!!!
    TRISEbits.RE1 = 1;         // input for the IRemote
    TRISBbits.RB4 = 1;         // ? TODO remove this one

    LATD = 0;                  // turn off LEDs
    TRISD = 0;                 // enable PORTD LED outputs

    // Timer 2  should be at 15151,51515 with the settings below
    TMR2 = 0;
    T2CON = 0; // clear all pre and postscaler to 0
    T2CONbits.T2CKPS0 = 0; // prescaler 1/1
    T2CONbits.T2CKPS1 = 0;
    T2CONbits.T2OUTPS = 2; // postscaler 1/3
    PR2 = 175;
    TMR2IP = 1; // high prio int
    TMR2IE = 1; // ena int
    TMR2ON = 1;

    irmp_init();

    RCONbits.IPEN   = 1; // enable interrupt priority
    INTCONbits.GIEH = 1;
    INTCONbits.GIEL = 1;

    // TODO: add a delay or this first message may get out garbled, clock and baudrate generator is not stable yet!
    printf("\r\nIRMPtest1 PIC4520!\r\n");

    // infinite loop, interrupts will blink PORTD pins and handle UART communications.
    while (1) {
        LATBbits.LATB0 = ~LATBbits.LATB0;

        if (irmp_get_data (&irmp_data))
        {
            // ir signal decoded, do something here...
            // irmp_data.protocol is the protocol, see irmp.h
            // irmp_data.address is the address/manufacturer code of ir sender
            // irmp_data.command is the command code
            // irmp_protocol_names[irmp_data.protocol] is the protocol name (if enabled, see irmpconfig.h)
            printf("proto %d addr %d cmd %d\n", irmp_data.protocol, irmp_data.address, irmp_data.command );
        }
    }

    return (EXIT_SUCCESS);
}



